%{ /* -*- makefile -*- */
/* 
 * We build a lexical analyzer that converts a Pascal source file to 
 * a beautifully highlighted HTML file now.
 */

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#include "colors.h"

#ifndef COMPRESSION
#define COMPRESSION 0
#endif

FILE *actin;
FILE *actout;

char *ownhead = "head.pas2html";
char *ownbottom = "bottom.pas2html";

%}
%s COMMENT1
%s COMMENT2
%s STRING

id [a-z][a-z0123456789_]*
wsnl [\t \n]+
ws   [\t ]+

comment1nospecial [^&<>----\\\n*)]
comment2nospecial [^&<>----\n}]
stringnospecial   [^&<>----\\\n']

%%
	/* This characters have to be recoded everywhere
	 * because the WWW-browsers interpret them in their
	 * own (here not wanted) way.
	 */
"&"	{ fprintf (actout, "&amp;"); }
"<"	{ fprintf (actout, "&lt;"); }
">"	{ fprintf (actout, "&gt;"); }
	/* All character as latin1 greater than dec192 or hexBF */
""	{ fprintf (actout, "&Agrave;"); }
""	{ fprintf (actout, "&Aacute;"); }
""	{ fprintf (actout, "&Acirc;"); }
""	{ fprintf (actout, "&Atilde;"); }
""	{ fprintf (actout, "&Auml;"); } 
""	{ fprintf (actout, "&Aring;"); }
""	{ fprintf (actout, "&AElig;"); }
""	{ fprintf (actout, "&Ccedil;"); }
""	{ fprintf (actout, "&Egrave;"); }
""	{ fprintf (actout, "&Eacute;"); }
""	{ fprintf (actout, "&Ecirc;"); } 
""	{ fprintf (actout, "&Euml;"); }  
""	{ fprintf (actout, "&Igrave;"); }
""	{ fprintf (actout, "&Iacute;"); }
""	{ fprintf (actout, "&Icirc;"); } 
""	{ fprintf (actout, "&Iuml;"); }  
	/* ""	{ fprintf (actout, "", yytext); } */
""	{ fprintf (actout, "&Ntilde;"); }
""	{ fprintf (actout, "&Ograve;"); }
""	{ fprintf (actout, "&Oacute;"); }
""	{ fprintf (actout, "&Ocirc;"); } 
""	{ fprintf (actout, "&Otilde;"); }
""	{ fprintf (actout, "&Ouml;"); }  
	/* ""	{ fprintf (actout, "", yytext); } */
""	{ fprintf (actout, "&Oslash;"); }
""	{ fprintf (actout, "&Ugrave;"); }
""	{ fprintf (actout, "&Uacute;"); }
""	{ fprintf (actout, "&Ucirc;"); } 
""	{ fprintf (actout, "&Uuml;"); }  
""	{ fprintf (actout, "&Yacute;"); }
""	{ fprintf (actout, "&THORN;"); }
""	{ fprintf (actout, "&szlig;"); }
""	{ fprintf (actout, "&agrave;"); }
""	{ fprintf (actout, "&aacute;"); }
""	{ fprintf (actout, "&acirc;"); } 
""	{ fprintf (actout, "&atilde;"); }  
""	{ fprintf (actout, "&auml;"); }
""	{ fprintf (actout, "&aring;"); }
""	{ fprintf (actout, "&aelig;"); }
""	{ fprintf (actout, "&ccedil;"); }
""	{ fprintf (actout, "&egrave;"); }
""	{ fprintf (actout, "&eacute;"); }
""	{ fprintf (actout, "&ecirc;"); } 
""	{ fprintf (actout, "&euml;"); }  
""	{ fprintf (actout, "&igrave;"); }
""	{ fprintf (actout, "&iacute;"); }
""	{ fprintf (actout, "&icirc;"); } 
""	{ fprintf (actout, "&iuml;"); }  
""	{ fprintf (actout, "&eth;"); }
""	{ fprintf (actout, "&ntilde;"); }
""	{ fprintf (actout, "&ograve;"); }
""	{ fprintf (actout, "&oacute;"); }
""	{ fprintf (actout, "&ocirc;"); } 
""	{ fprintf (actout, "&otilde;"); }  
""	{ fprintf (actout, "&ouml;"); }
	/* ""	{ fprintf (actout, "", yytext); } */
""	{ fprintf (actout, "&oslash;"); }
""	{ fprintf (actout, "&ograve;"); }
""	{ fprintf (actout, "&oacute;"); }
""	{ fprintf (actout, "&ocirc;"); } 
""	{ fprintf (actout, "&ouml;"); }  
""	{ fprintf (actout, "&yacute;"); }
""	{ fprintf (actout, "&thorn;"); }
""	{ fprintf (actout, "&yuml;"); }

<STRING>\'	{ 
		  fprintf (actout, "%s</FONT>", yytext);
		  BEGIN 0;
		}
<STRING>\n	{
		    fprintf (actout, "%s", yytext);
		}
<STRING>{stringnospecial}+	{ fprintf (actout, "%s", yytext); }

<COMMENT1>("(*"|"{")	{ fprintf (actout, "%s", yytext); }
<COMMENT1>\n	{
		    fprintf (actout, "%s", yytext);
		}
<COMMENT1>{comment1nospecial}+	{ fprintf (actout, "%s", yytext); }
<COMMENT1>"*"/[^)]	{ fprintf (actout, "%s", yytext); }
<COMMENT1>"*)"	{
		  fprintf (actout, "%s</FONT>", yytext);
		  BEGIN 0; 
		}
<COMMENT2>[*)]  {
		  fprintf (actout, "%s</FONT>", yytext);
		}

<COMMENT2>{comment2nospecial}* { fprintf (actout, "%s", yytext); }
<COMMENT2>"}"	{ 
		  fprintf (actout, "%s</FONT>", yytext);
		  BEGIN 0; 
		}
\'	{
	  BEGIN STRING;
	  fprintf (actout, "<FONT COLOR=%s>%s", stringcolor, yytext);
	}
"(*"	{
	  BEGIN COMMENT1;
	  fprintf (actout, "<FONT COLOR=%s>%s", commentcolor, yytext);
	}
"{"	{
	  BEGIN COMMENT2;
	  fprintf (actout, "<FONT COLOR=%s>%s", commentcolor, yytext);
	}


"var"|"const"|"type"	{
		  fprintf (actout, "<STRONG><FONT COLOR=%s>%s</FONT></STRONG>",
			   definelinecolor, yytext);
		}
"begin"|"end"|"program"	{
		  fprintf (actout, "<STRONG><FONT COLOR=%s>%s</FONT></STRONG>",
			   keywordcolor, yytext);
		}

("procedure"|"function"){wsnl}{id}	{
		  fprintf (actout, "<FONT COLOR=%s><STRONG>%s</STRONG></FONT>",
			   keywordcolor, yytext);
		}		  
"while"|"for"|"do"|"repeat"|"until"|"case"|"to"|"of"|"if"|"not"|"then"|"and"|"or"	{
		  fprintf (actout, "<FONT COLOR=%s>%s</FONT>",
			   keywordcolor, yytext);
		}

{ws}		{ 
		 fprintf (actout, "%s", yytext); 
		  /* let whitespaces like they are */
		}

[a-z_][a-z_0123456789]*	{
		 fprintf (actout, "%s",yytext);
		}

.		{ 
		 fprintf (actout, "%s",yytext); 
		}
%%

int insert (FILE * outfile, char *filename);
/* 
 * inserts the file "filename" in the current directory (or if given the 
 * complete path this file) into the output file.
 */

static char *prog;
static void usage() 
{
  fprintf (stderr, 
           "usage: %s -V [-t title] [-w width] [file_to_convert ...]\n", 
           prog);
  exit(3);
}

int
main(int argc, char *argv[])
{
  char *outfilename = (char *) NULL;
  int  i            = 0;
  int  ch           = 0;
  int  width        = 80;
  int  rc           = 0;
  char *title       = (char *) NULL;
  char *pt          = (char *) NULL;

  char gzipcmd[10];

  pt = strrchr(argv[0], '/');
  prog = (pt)? ++pt : argv[0]; /* basename */
  while (-1 != (ch = getopt(argc, argv, "h?t:w:V")))
    {
      switch(ch) 
	{
	case 't':
	  title = optarg;
	  break;
	case 'w':
	  if (1 != sscanf(optarg, "%d", &width))
	    { usage(); }
	  break;
	case 'V':
	  fprintf(stderr, "pas2html version %s%s\n",
		  VERSION, 
                  COMPRESSION ? "" : " (no compression)");
	  exit(0);
	default:
	  usage();
	}
    }
  argc -= optind;
  argv += optind;

  if (0 == argc) 
    {
      actin  = stdin;
      actout = stdout;
      
      if ((pt = getenv("PATH_TRANSLATED")) 
	  && getenv("GATEWAY_INTERFACE")) 
	{
	  /* CGI */
	  if (!title)
	    { title = pt; }
	  actin = fopen (pt, "r");
	  if (!actin) 
	    {
	      fprintf (actout, "Content-Type: text/html\n\n");
	      fprintf (actout, "<HTML><HEAD><TITLE>error in CGI '%s'", prog);
	      fprintf (actout, "</TITLE></HEAD>\n");
	      fprintf (actout, "<BODY><H1>error in CGI program '%s': ", prog);
	      fprintf (actout, "cannot read file '%s'</H1></BODY>\n", pt);
	      fprintf (actout, "</HTML>\n");
	      exit(1);
	    }
	  fprintf (actout, "Content-Type: text/html\n");
          if (COMPRESSION)
	    {
	      /* should we compress output with gzip? */
	      do 
		{
		  if (!(pt = getenv("HTTP_ACCEPT_ENCODING")))
		    { break; } /* no header Accept-Encoding: */
		  if (!strstr(pt, "gzip"))
		    { break; } /* browser doesn't understand gzip format */
		  if ((pt = getenv("REMOTE_ADDR")) 
		      && (!strncmp(pt, "127", 3)))
		    { break; } /* never compress to local client */
		  /* OK, compress output with gzip */
		  fprintf(actout, "Content-Encoding: x-gzip\n\n");
		  fflush(actout);
		  sprintf(gzipcmd, "gzip -%d", COMPRESSION);
		  if (!(actout = (FILE *) popen(gzipcmd, "w")))
		    { exit(4); } /* cannot fork, gzip not found, ... */
		} 
	      while (0);
	    }
	} /* end of CGI section */
      fprintf (actout, "\n<HTML>\n<HEAD>\n");
      fprintf (actout, "<TITLE>%s", title ? title : "stdin");
      fprintf (actout, "</TITLE>\n</HEAD>\n\n");
      fprintf (actout, "<BODY BGCOLOR=\"%s\">\n", bgcolor);
      (void) insert (actout, ownhead);
      fprintf (actout, "<PRE WIDTH=\"%d\">", width);
  
      yyin  = actin;
      yyout = actout;

      BEGIN 0;
      yylex();
      
      fprintf (actout,"</PRE>\n</BODY>\n\n</HTML>\n");
    } 
  else 
    for (i = 0; i < argc; i++) 
      {
	actin = fopen (argv[i], "r");
	if (!actin) 
	  {
	    fprintf (stderr, "%s: cannot read file '%s'\n", prog, argv[i]);
	    rc = rc < 1 ? 1 : rc;
	    continue;
	  }
      
	outfilename = malloc (sizeof (char)*strlen (argv[i]) + 6);
	strcat (strcpy (outfilename, argv[i]), ".html");
	actout = fopen (outfilename, "w");
	if (!actout) 
	  {
	    fprintf (stderr, "%s: cannot write file '%s'\n", prog, outfilename);
	    rc = rc < 2 ? 2 : rc;
	    continue;
	  }
	outfilename[strlen(outfilename) - 5] = '\0'; /* chop '.html' suffix */
	fprintf (actout, "<HTML>\n<HEAD>\n");
	fprintf (actout, "<TITLE>%s", outfilename);
	fprintf (actout, "</TITLE>\n</HEAD>\n\n");
	fprintf (actout, "<BODY BGCOLOR=\"%s\">\n", bgcolor);
	(void) insert (actout, ownhead);
	fprintf (actout, "<PRE WIDTH=\"%d\">", width);

	yyin  = actin;
	yyout = actout;

	BEGIN 0;
	yylex ();

	fprintf (actout, "</PRE>\n");
	(void) insert (actout, ownbottom);
	fprintf (actout, "</BODY>\n\n</HTML>\n");
	free (outfilename);
	fclose (actin);
	fclose (actout);
      }
  return rc;
  /* unreachable, but suppresses compiler warning. */
  yyunput(0, NULL);
}

int
insert (FILE * outfile, char *filename)
{
  int c;
  FILE * infile;
 
  infile = fopen (filename, "r");
  if (NULL != infile)
    {
      c = fgetc (infile);
      while (EOF != c)
        { 
          fputc (c, outfile);
          c = fgetc (infile);
        }
      fclose (infile);
      return 1;
    }
  else
    { return 0; }
}
